/*	File:	msg.h
	Author:	Alex Fedosov
	Date:	September 28, 2000
	Description:	definitions of game messages and related objects
*/

#ifndef __DMZ_MSG_H__
#define __DMZ_MSG_H__

#include "common/types.h"
#include "common/object.h"

/* this enumeration defines several message types as used by the game engine */
typedef enum {	
				FILEXFER,	/* file transfer -- server sends a file */
				GAMEINFO,	/* information about the game */
				OBJECT,		/* object update -- server sends the updated object */
				DEATH,		/* death of a salesman */
				END,		/* player quit, don't show but preserve position */
				BEGIN,		/* player connects, set up initial position, etc. */
				MOVE,		/* used to request and report movement */
				CHAT,		/* chat -- basic method of communication */
				ATTACK,		/* to request and report attack */
				ACTION,		/* a player wants to interact with an object */
				UPDATE		/* generic update of an object */
} MSGTYPE;


struct dmz_mess_UPDATE
{
	WORD16 status;			/* new status */
	WORD16 life;			/* new life */
	WORD16 score;			/* new score */
	WORD16 frags;			/* new frags */
	WORD16 ammo;			/* new ammo */
};

/* structure for a move message. client fills one of these out requesting a move.
   if move request is denied, server does not respond. otherwise, the server broadcasts
   the same message to all the clients, including the original requested. only then the
   movement is valid and character can be re-drawn.

   type:	MOVE
   objid:	id of the object that is moving
   objtype:	type of the object that is moving (player, NPC, etc.)
*/
struct dmz_mess_MOVE
{
	WORD32 new_x;			/* requested new x position (cell index into map) */
	WORD32 new_y;			/* new y position (cell index) */
	WORD16 direction;		/* direction the player is facing */
};

/* message used whenever a character enters the game. 

   type:	BEGIN
   objid:	id of the object entering the game (0 to request a new one)
   objtype:	type of object (should be PLAYER)
*/
struct dmz_mess_BEGIN
{
	WORD32 init_x;		/* initial x position */
	WORD32 init_y;		/* initial y position */
	WORD16 init_dir;	/* initial direction */
	WORD16 bitmap;		/* character's bitmap */
	char name[NAMELEN];		/* player name */
};

/* message used whenever a character leaves the game. there is no separate
   structure, as the format is the same as with MOVE. 

   type:	END
   objid:	id of the object leaving the game 
   objtype:	type of object
*/

/* message used whenever a character interacts with another object. 

   type:	ACTION
   objid:	id of the player
   objtype:	type of object (should be PLAYER)
*/
struct dmz_mess_ACTION
{
	WORD16 object;		/* id of the object with which we are interacting */
	WORD16 type;		/* interaction type */
};

/* this message is used whenever a player wants to talk to another player
   NOTE: currently broadcast to everyone in the game

   type:	CHAT
   objid:	id of the object talking
   objtype:	not used
*/
struct dmz_mess_CHAT
{
	char text[MAXMSGDATA];	/* the actual message */
};

/* message used whenever a player (or monster) is trying to attack someone. just like
   with movement, a client sends one of these to request attack. server determines
   whether it succeeded (performing necessary checks) and broadcasts the result.
   
   type:	ATTACK
   objid:	id of the object attacking
   objtype:	not used
*/
struct dmz_mess_ATTACK
{
	WORD16 targetid;		/* who is being attacked? */
	WORD16 success;			/* 0 = miss, nonzero = hit */
	WORD16 method;			/* missile or HTH */
	WORD16 direction;		/* current direction facing */
};

/* message used whenever a character dies. there is no separate structure,
   as no extra info is needed.

   type:	DEATH
   objid:	id of the object that just died
   objtype:	type of object
*/

/* structure for the object message. client fills one out to request an object and sets 
   objsize to 0 to request download. server copies the object data into the message,
   updates the object size and sends it back.

   type:	OBJECT
   objid:	id of the object transferred
   objtype:	type of object
*/
struct dmz_mess_OBJECT
{
	struct dmz_object obj;	/* object data */
};

/* message used for file transfer between server and client. client fills out a request
   with filesize set to 0. upon receipt, the server puts up to OBJSIZE bytes into data,
   and sends it back. complete flag is set if this is the last chunk.

   type:	FILEXFER
   objid:	file id of the file being transferred
   objtype:	not used
*/
struct dmz_mess_FILEXFER
{
	WORD16 complete;		/* flag indicating whether transfer is complete */
	int filesize;			/* size of this piece */
	char filedata[MAXMSGDATA];	/* file data */
};

/* this message is used to transmit general information about the game settings,
   such as number of players, resource file used, etc.

   type:	GAMEINFO
   objid:	not used
   objtype:	not used
*/
struct dmz_mess_GAMEINFO
{
	WORD16 players;
	WORD16 map_x;
	WORD16 map_y;
	char mapfile[32];
};

/* generic message structure */
struct dmz_message
{
	MSGTYPE type;			/* type of message */
	WORD16 objid;			/* unique object id, used for all messages */

	union
	{
		struct dmz_mess_MOVE move;
		struct dmz_mess_BEGIN begin;
		struct dmz_mess_CHAT chat;
		struct dmz_mess_ATTACK attack;
		struct dmz_mess_OBJECT object;
		struct dmz_mess_FILEXFER filexfer;
		struct dmz_mess_GAMEINFO gameinfo;
		struct dmz_mess_ACTION action;
		struct dmz_mess_UPDATE update;
	} u;
};

typedef struct dmz_message* HDMZMSG;

#endif
