/*	File:	object.h
	Author:	Alex Fedosov, Jeff Malch
	Date:	February 5, 2001
	Description:	definitions of game objects
*/

#ifndef __DMZ_OBJECT_H__
#define __DMZ_OBJECT_H__

#include "common/types.h"
#include "common/const.h"
#include <time.h>

/* enumeration for different types of objects */
typedef enum OBJTYPE
{
				PLAYER,		/* player character (human or AI) */
				MONSTER,	/* AI-controlled beastie */
				NPC,		/* non-player character */
				ITEM,		/* game item -- weapon, armor, etc. */
				TERRAIN,	/* an element of terrain -- rock, tree, etc. */
				MISSILE		/* flying thingie */
} OBJTYPE;

/* state of object (player) */
typedef enum OBJSTATE
{
				IDLE,		/* not dead */
				MOVING,		/* going somewhere */
				SLEEPING,	/* in perfect hybernation */
				DEAD,		/* dead! */
				INVISIBLE	/* should not be displayed */
} OBJSTATE;

/* actions that can be done with objects */
typedef enum ACTIONS
{
				PICKUP,		/* pick it up */
				DROP,		/* drop it */
				OPEN,		/* try to break it */
				TALK,		/* talk to it? */
				USE			/* generic */
} ACTIONS;

/* jeff added 31201 */
typedef enum ITEMTYPE
{
				ARMORUPGRADE = 1,
				DAMAGEUPGRADE = 2,
				AMMOUPGRADE = 3,
				SPEEDUPGRADE = 4,
				BOOTY = 5
} ITEMTYPE;

/* self-explanatory directions */
typedef enum DIRECTION
{
				NORTH = 0,
				NORTHEAST = 1,
				EAST = 2,
				SOUTHEAST = 3,
				SOUTH = 4,
				SOUTHWEST = 5,
				WEST = 6,
				NORTHWEST = 7
} DIRECTION;

/* player structure, same as monster */
struct dmz_PLAYER
{
	WORD16 life;					/* amount of life */
	WORD16 ammo;					/* amount of ammo */
	WORD16 level;					/* current level */
	WORD16 frags;					/* frag count */
	WORD16 direction;				/* direction player is facing */
	WORD16 frame;					/* frame number */
	WORD16 cur_item;				/* active item (weapon, etc.) */
	WORD16 hth_dam;					/* hand-to-hand damage */
	WORD16 mis_dam;					/* missile damage */
};

struct dmz_MISSILE
{
	WORD16 range;					/* time to live of missile */
	WORD16 damage;					/* amount of damage missile does */
	WORD16 direction;				/* direction of projectile */
	WORD16 owner;					/* player that launched missile */
	WORD16 exp_frame;				/* frame of explosion animation */
	WORD16 exp_ttl;					/* time to live of explosion */
};

struct dmz_NPC
{
	WORD16 npc_dir;					/* direction facing */
	WORD16 npc_walk;				/* walking animation sequence */
};

struct dmz_ITEM
{
	WORD16 item_arm;				/* armor */
	WORD16 item_dam;				/* damage */
	WORD16 item_ammo;
	WORD16 item_frame;
	WORD16 kind;
};

struct dmz_TERRAIN
{
	WORD16 ter_hp;					/* hitpoints of the terrain */
};

/* generic game object: can be player, item, terrain. items and terrain
   elements do not need any extra space. players and NPCs, however, need
   such things as inventory, death scenes, skills, etc. for this purpose
   they have a union in addition to the generic fields. */
struct dmz_object
{
	WORD16 objid;			/* unique object id */
	WORD16 objtype;			/* object type */
	WORD16 bitidx;			/* index into the bitmap table giving object's bitmap */
	WORD16 deadbit;			/* "dead" object bitmap */
	WORD16 velocity;		/* object's velocity */
	WORD16 score;			/* "value" of the object */
	WORD16 inv[MAX_INV];	/* inventory, as object id's */
	OBJSTATE status;		/* dead or alive? */
	WORD32 x;				/* object coordinates in pixels */
	WORD32 y;
	int width;				/* bitmap width and height */
	int height;
	char name[32];			/* object name */
	time_t tstamp;			/* time stamp for regeneration */

	union
	{
		struct dmz_PLAYER player; /* monster has same structure */
		struct dmz_NPC npc;
		struct dmz_MISSILE missile;
		struct dmz_ITEM item;
		struct dmz_TERRAIN terrain;
	} u;
};

typedef struct dmz_object* HOBJECT;

GLOBAL HOBJECT dmz_objects[MAXOBJECTS];

WORD16 dmz_new_object(WORD16 objid, WORD16 objtype, WORD32 init_x, WORD32 init_y);
void dmz_delete_object(WORD16 objid);

/* update the status of an object */
void dmz_status_update(WORD16 obj, OBJSTATE status);

/* initialize object system */
void dmz_object_init(void);
void dmz_object_destroy(void);

#endif
